# ! Callback Test Setup Version 585 (Millipede04)
import math,random

# Version 584
# Added callbacks for the yRotates of each of the four segments of my basic millipede.
# There are debug prints in all the yRotate callbacks to help me check the maths
# There's no error checking so if you try to run this script on anything except my Millipede04.cr2 it'll crash
# 
# Next step is probably to try and do the leg cycles in the callbacks too...


#--------------------------------------------------------------------
# Initialize the precalculated arrays
#--------------------------------------------------------------------
def InitYRots():
	global ARRAY_SIZE
	global yRotArray
	global xTranArray
	global zTranArray
	ARRAY_SIZE=360
	yRotArray = range(ARRAY_SIZE)
	xTranArray = range(ARRAY_SIZE)
	zTranArray = range(ARRAY_SIZE)
	for i in range(ARRAY_SIZE):
		if i <= 1:
			yRotArray[i] = 0
			xTranArray[i] = 0
			zTranArray[i] = 0
		else:
			yRotArray[i] = (yRotArray[i-1] + (30 * (random.random()-0.5))) % 360 
			xTranArray[i] = xTranArray[i-1] + (onestep*math.sin(math.radians(yRotArray[i])))
			zTranArray[i] = zTranArray[i-1] + (onestep*math.cos(math.radians(yRotArray[i])))
		print "Frame"+str(i)+": yR="+str(yRotArray[i])+", xT="+str(xTranArray[i])+", zT="+str(zTranArray[i])

#---------------------------------------------------------------------
# return the next precalculated y rotation
#---------------------------------------------------------------------
def GetYRot(frame):
	global yRotArray
	global ARRAY_SIZE
	if frame >ARRAY_SIZE:
		return yRotArray[ARRAY_SIZE-1]
	else:
		return yRotArray[frame]
#---------------------------------------------------------------------
# return the next precalculated x translation
#---------------------------------------------------------------------
def GetXTran(frame):
	global xTranArray
	global ARRAY_SIZE
	if frame >ARRAY_SIZE:
		return xTranArray[ARRAY_SIZE]
	else:
		return xTranArray[frame]
#---------------------------------------------------------------------
# return the next precalculated z translation
#---------------------------------------------------------------------
def GetZTran(frame):
	global zTranArray
	global ARRAY_SIZE
	if frame >ARRAY_SIZE:
		return zTranArray[ARRAY_SIZE]
	else:
		return zTranArray[frame]


# Callback for the Body "yRotate" parameter
def cheapskatesYRotateCallback(parm,value):
	global yRot
	global yRotFrame
	curframe = poser.Scene().Frame()
	if curframe != yRotFrame:
		yRot = GetYRot(curframe)
		yRotFrame = curframe
		print "\n"+str(curframe)+": Body="+str(yRot)
	return yRot
# Callback for the Body "xTran" parameter
def cheapskatesXTranCallback(parm,value):
	global xPos
	global xPosFrame
	curframe = poser.Scene().Frame()
	if curframe != xPosFrame:
		xPos = GetXTran(curframe)
		xPosFrame = curframe
	return value+xPos
# Callback for the Body "zTran" parameter
def cheapskatesZTranCallback(parm,value):
	global zPos
	global zPosFrame
	curframe = poser.Scene().Frame()
	if curframe != zPosFrame:
		zPos = GetZTran(curframe)
		zPosFrame = curframe
	return value+zPos
	

# Callback for the Segment_01 "yRotate" parameter
def cheapskatesSeg1YRotCallback(parm,value):
	global yRotSeg1
	global yRotFrameSeg1
	curframe = poser.Scene().Frame()
	if curframe >= 2:
		if curframe != yRotFrameSeg1:
			yRotFrameSeg1 = curframe
			yRotSeg1 = GetYRot(curframe-1) - GetYRot(curframe)
			print str(curframe)+": Seg1="+str(yRotSeg1)
		return yRotSeg1
	else:
		return 0.0
# Callback for the Segment_02 "yRotate" parameter
def cheapskatesSeg2YRotCallback(parm,value):
	global yRotSeg2
	global yRotFrameSeg2
	curframe = poser.Scene().Frame()
	if curframe >= 3:
		if curframe != yRotFrameSeg2:
			yRotFrameSeg2 = curframe
			yRotSeg2 = GetYRot(curframe-2) - GetYRot(curframe-1)
			print str(curframe)+": Seg2="+str(yRotSeg2)
		return yRotSeg2
	else:
		return 0.0
# Callback for the Segment_03 "yRotate" parameter
def cheapskatesSeg3YRotCallback(parm,value):
	global yRotSeg3
	global yRotFrameSeg3
	curframe = poser.Scene().Frame()
	if curframe >= 4:
		if curframe != yRotFrameSeg3:
			yRotFrameSeg3 = curframe
			yRotSeg3 = GetYRot(curframe-3) - GetYRot(curframe-2)
			print str(curframe)+": Seg3="+str(yRotSeg3)
		return yRotSeg3
	else:
		return 0.0
# Callback for the Segment_04 "yRotate" parameter
def cheapskatesSeg4YRotCallback(parm,value):
	global yRotSeg4
	global yRotFrameSeg4
	curframe = poser.Scene().Frame()
	if curframe >= 5:
		if curframe != yRotFrameSeg4:
			yRotFrameSeg4 = curframe
			yRotSeg4 = GetYRot(curframe-4) - GetYRot(curframe-3)
			print str(curframe)+": Seg4="+str(yRotSeg4)
		return yRotSeg4
	else:
		return 0.0

# Need to recalculate yRot,XTran, and zTran each frame based on calculated ('keyed', upper figure on dial) value from previous frame, NOT the dialed ('natural', lower figure on dial)
# This (i.e. using globals) seemed the easiest way to do it, at least for my first attempt
global yRot
global yRotFrame
global xPos
global xPosFrame
global zPos
global zPosFrame
global yRotSeg1
global yRotFrameSeg1
global yRotSeg2
global yRotFrameSeg2
global yRotSeg3
global yRotFrameSeg3
global yRotSeg4
global yRotFrameSeg4

# Arrays to hold precalulated values
global ARRAY_SIZE
global yRotArray
global xTranArray
global zTranArray

# These are just to avoid having to do 'body.Parameter("yRotate")' etc in the callbacks. There are others I could add too.
global parmXTran
global parmZTran
global onestep

# Initialize a few things
fig=poser.Scene().CurrentFigure()
body=fig.ActorByInternalName("BODY")
seg1=fig.ActorByInternalName("seg1")
seg2=fig.ActorByInternalName("seg2")
seg3=fig.ActorByInternalName("seg3")
seg4=fig.ActorByInternalName("seg4")
yRot=0
yRotFrame=0
xPos=0
xPosFrame=0
zPos=0
zPosFrame=0
yRotSeg1=0
yRotFrameSeg1=0
yRotSeg2=0
yRotFrameSeg2=0
yRotSeg3=0
yRotFrameSeg3=0
yRotSeg4=0
yRotFrameSeg4=0
parmYRot=body.Parameter("yRotate")
parmXTran=body.Parameter("xTran")
parmZTran=body.Parameter("zTran")
onestep=0.1

# Initialize the precalculated array
InitYRots()
	
# Kick off the callbacks
print "Setting up the Body 'yRotate' callback..."
body.Parameter("yRotate").SetUpdateCallback(cheapskatesYRotateCallback)
print "Setting up the Body 'xTran' callback..."
body.Parameter("xTran").SetUpdateCallback(cheapskatesXTranCallback)
print "Setting up the Body 'zTran' callback..."
body.Parameter("zTran").SetUpdateCallback(cheapskatesZTranCallback)
print "Setting up the Segment_01 'yRotate' callback..."
seg1.Parameter("yRotate").SetUpdateCallback(cheapskatesSeg1YRotCallback)
print "Setting up the Segment_02 'yRotate' callback..."
seg2.Parameter("yRotate").SetUpdateCallback(cheapskatesSeg2YRotCallback)
print "Setting up the Segment_03 'yRotate' callback..."
seg3.Parameter("yRotate").SetUpdateCallback(cheapskatesSeg3YRotCallback)
print "Setting up the Segment_04 'yRotate' callback..."
seg4.Parameter("yRotate").SetUpdateCallback(cheapskatesSeg4YRotCallback)
print "...done."
